import json
import requests

from http import HTTPStatus
from typing import Optional, Dict, Any


class DuitkuResult:
    def __init__(
        self,
        status_code=HTTPStatus.OK,
        message=None,
        raw_request="",
        raw_response=""
    ):
        """
        Initializes a DuitkuResult object.

        :param status_code: The HTTP status code of the response.
        :param message: The JSON response message.
        :param raw_request: The raw request body.
        :param raw_response: The raw response body.
        """
        self.status_code = status_code
        self.message = message
        self.raw_request = raw_request
        self.raw_response = raw_response

class DuitkuClient:
    SandboxV2BaseURL = 'https://sandbox.duitku.com/webapi/api'
    ProductionV2BaseURL = 'https://passport.duitku.com/webapi/api'
    SandboxPOPBaseURL = 'https://api-sandbox.duitku.com/api'
    ProductionPOPBaseURL = 'https://api-prod.duitku.com/api'
    SandboxEnv = 'sandbox'
    ProductionEnv = "production"
    def __init__(
        self, 
        merchant_code=None,
        api_key=None,
        environment="sandbox"
    ):
        """
        Initializes a DuitkuClient object.

        :param merchant_code: The merchant code obtained from Duitku.
        :param api_key: The API key obtained from Duitku.
        :param environment: The environment to use. Can be either "sandbox" or "production". Default is "sandbox".
        """
        self.merchant_code = merchant_code
        self.api_key = api_key
        self.environment = environment

    def get_v2_base_url(self):
        """
        Gets the base URL for the Duitku V2 API.

        :return: The base URL for the Duitku V2 API.
        """
        if self.environment == "sandbox":
            return self.SandboxV2BaseURL
        else:
            return self.ProductionV2BaseURL
        
    def get_pop_base_url(self):
        """
        Gets the base URL for the Duitku POP API.

        :return: The base URL for the Duitku POP API.
        """
        if self.environment == "sandbox":
            return self.SandboxPOPBaseURL
        else:
            return self.ProductionPOPBaseURL
        
    def send_api_request(
        self,
        method: str,
        url: str,
        req_body: Optional[Dict[str, Any]],
        header_params: Dict[str, str] = None
    ) -> DuitkuResult:
        """
        Sends a request to the Duitku API.

        :param method: The HTTP method to use.
        :param url: The URL to send the request to.
        :param req_body: The request body to send.
        :param header_params: Additional headers to include.
        :return: A DuitkuResult object containing the response.
        """
        headers = {"Content-Type": "application/json"}
        if header_params is not None:
            headers.update(header_params)

        if req_body is not None:
            data = json.dumps(req_body)
        else:
            data = None

        response = requests.request(
            method,
            url,
            headers=headers,
            data=data,
        )
        return self._handle_response(response)
    
    def _handle_response(self, response: requests.Response) -> DuitkuResult:
        """
        Handles the HTTP response from the Duitku API and returns a DuitkuResult object.

        :param response: The HTTP response object received from the API request.
        :return: A DuitkuResult object containing the status code, raw request, raw response,
                and parsed JSON message if available; otherwise, the raw text message.
        """
        result = DuitkuResult(
            status_code=response.status_code,
            raw_request=response.request.__dict__,
            raw_response=response.raw._original_response.__dict__,
        )
        try:
            if response.text:
                result.message = response.json()
        except json.decoder.JSONDecodeError:
            result.message = response.text
        return result
        


import unittest
import os

import Duitku

from http import HTTPStatus
from dotenv import load_dotenv
from datetime import datetime

load_dotenv()

class TestPayment(unittest.TestCase):
    duitku = Duitku.Duitku()

    client = duitku.client
    client.merchant_code = os.getenv('MERCHANT_CODE')
    client.api_key = os.getenv('API_KEY')

    def test_get_payment_methods_success(self):
        request_get_payment_methods = {
            "amount": 10001,
            "datetime": datetime.now().strftime("%Y-%m-%d %H:%M:%S"),
        }

        result = self.duitku.payment.get_methods(request_get_payment_methods)
        self.assertEqual(result.status_code, HTTPStatus.OK)

        self.assertIsInstance(result.message['paymentFee'], list)
        self.assertIsInstance(result.message['paymentFee'][0]['paymentMethod'], str)
        self.assertIsInstance(result.message['paymentFee'][0]['paymentName'], str)
        self.assertIsInstance(result.message['paymentFee'][0]['paymentImage'], str)
        self.assertIsInstance(result.message['paymentFee'][0]['totalFee'], str)
        self.assertEqual(result.message['responseCode'], '00')
        self.assertEqual(result.message['responseMessage'], 'SUCCESS')

if __name__ == '__main__':
    unittest.main()


from dataclasses import dataclass, field
from typing import List, Optional

@dataclass
class ItemDetails:
    name: str
    quantity: int
    price: int

@dataclass
class CustomerDetail:
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    email: Optional[str] = None
    phoneNumber: Optional[str] = None

@dataclass
class CreditCardDetail:
    acquirer: str
    binWhitelist: List[str]

@dataclass
class CreateInvoiceRequest:
    paymentAmount: int
    merchantOrderId: str
    productDetails: str
    email: str
    additionalParam: Optional[str] = None
    merchantUserInfo: Optional[str] = None
    customerVaName: str = ''
    phoneNumber: Optional[str] = None
    itemDetails: Optional[List[ItemDetails]] = field(default_factory=list)
    customerDetail: Optional[CustomerDetail] = None
    callbackUrl: str = ''
    returnUrl: str = ''
    expiryPeriod: Optional[int] = None
    paymentMethod: str = ''
    creditCardDetail: Optional[CreditCardDetail] = None

@dataclass
class CreateInvoiceResponse:
    merchantCode: str
    reference: str
    paymentUrl: str
    amount: str
    statusCode: str
    statusMessage: str


import hashlib

from ..client import DuitkuClient, DuitkuResult

class TransactionService:
    def __init__(self, client: DuitkuClient):
        self.client = client
        self.base_url = self.client.get_v2_base_url()

    def create(
        self,
        request: dict
    ) -> DuitkuResult:
        """
        Create a new transaction.

        :param request: A dictionary containing the request parameters.
        :return: A DuitkuResult object containing the response parameters.
        """
        path = "/merchant/v2/inquiry"
        request['merchantCode'] = self.client.merchant_code
        request['signature'] = self._generate_transaction_signature(request.get('merchantOrderId', '') +  str(request.get('paymentAmount')))
        url = self.base_url + path

        result = self.client.send_api_request(
            method="POST",
            url=url,
            req_body=request,
            header_params=None
        )
        return result
    
    def get_status(
        self,
        request: dict
    ) -> DuitkuResult:
        """
        Get the status of a transaction.

        :param request: A dictionary containing the request parameters.
        :return: A DuitkuResult object containing the response parameters.
        """
        path = "/merchant/transactionStatus"
        request['merchantCode'] = self.client.merchant_code
        request['signature'] = self._generate_transaction_signature(request['merchantOrderId'])
        url = self.base_url + path

        result = self.client.send_api_request(
            method="POST",
            url=url,
            req_body=request,
            header_params=None
        )
        return result
    
    def _generate_transaction_signature(self, parameter: str) -> str:
        """
        Generates a transaction signature using the given parameter, merchant code, and API key.

        :param parameter: A string containing the request parameters.
        :return: A string representing the generated signature in hexadecimal format.
        """
        combined_str = self.client.merchant_code + parameter + self.client.api_key
        return hashlib.md5(combined_str.encode()).digest().hex()


import hmac
import hashlib
import time
from ..client import DuitkuClient, DuitkuResult

class InvoiceService:
    def __init__(self, client: DuitkuClient):
        """
        Initializes the InvoiceService with a given DuitkuClient.

        :param client: An instance of DuitkuClient used to handle API requests.
        """
        self.client = client
        self.base_url = self.client.get_pop_base_url()

    def create(
        self, 
        request: dict,
    ) -> DuitkuResult:
        """
        Create a new invoice using the given request.
        
        The request should contain at least the following parameters:
        
        - paymentAmount
        - merchantOrderId
        - productDetails
        - email
        - callbackUrl
        - returnUrl
        
        The response will contain the following parameters:
        
        - merchantCode
        - reference
        - paymentUrl
        - amount
        - statusCode
        - statusMessage
        
        :param request: A dictionary containing the request parameters
        :return: A DuitkuResult object containing the response parameters
        """
        path = "/merchant/createInvoice"
        headers = {
            "x-duitku-merchantcode": self.client.merchant_code,
            "x-duitku-timestamp": str(int((round(time.time() * 1000)))),
        }
        headers["x-duitku-signature"] = self._generate_invoice_signature(headers["x-duitku-timestamp"])
        url = self.base_url + path

        result = self.client.send_api_request(
            method="POST",
            url=url,
            req_body=request,
            header_params=headers
        )
        return result
    
    def _generate_invoice_signature(self, timestamp: str) -> str:
        """
        Generates a signature for invoice creation using the given timestamp and the merchant's API key.

        :param timestamp: A string representing the current timestamp in milliseconds
        :return: A string representing the signature
        """
        str_signature = self.client.merchant_code + timestamp
        return hmac.new(
            self.client.api_key.encode(),
            str_signature.encode(),
            hashlib.sha256
        ).hexdigest()



import hashlib

from ..client import DuitkuClient, DuitkuResult

class PaymentService:
    def __init__(self, client: DuitkuClient):
        """
        Initializes the PaymentService with a given DuitkuClient.

        :param client: An instance of DuitkuClient used to handle API requests.
        """
        self.client = client
        self.base_url = self.client.get_v2_base_url()

    def get_methods(
        self,
        request: dict,
    ) -> DuitkuResult:
        """
        Get a list of payment methods.

        :param request: A dictionary containing the request parameters
        :return: A DuitkuResult object containing the response parameters
        """
        path = "/merchant/paymentmethod/getpaymentmethod"
        request['merchantCode'] = self.client.merchant_code
        request['signature'] = self._generate_payment_signature(str(request['amount']) + request['datetime'])
        url = self.base_url + path
        response = self.client.send_api_request(
            method="POST",
            url=url,
            req_body=request,
        )
        return response

    def _generate_payment_signature(self, paramter: str) -> str:
        """
        Generates a signature for the payment method request using the given parameter and the merchant's API key.

        :param paramter: A string containing the request parameters
        :return: A string representing the signature
        """
        combined_str = self.client.merchant_code + paramter + self.client.api_key
        return hashlib.sha256(combined_str.encode()).digest().hex()


from dataclasses import dataclass
from typing import List, Optional

@dataclass
class ItemDetails:
    name: str
    quantity: int
    price: int

@dataclass
class CustomerDetail:
    firstName: Optional[str] = None
    lastName: Optional[str] = None
    email: Optional[str] = None
    phoneNumber: Optional[str] = None

@dataclass
class OvoPaymentDetails:
    paymentType: str
    amount: int

@dataclass
class Ovo:
    paymentDetails: List[OvoPaymentDetails]

@dataclass
class Shopee:
    promo_ids: str
    useCoin: str

@dataclass
class AccountLink:
    credentialCode: str
    ovo: Ovo
    shopee: Shopee

@dataclass
class CreditCardDetail:
    acquirer: str
    binWhitelist: List[str]

@dataclass
class CreateTransactionRequest:
    merchantCode: str
    paymentAmount: int
    merchantOrderId: str
    productDetails: str
    email: str
    paymentMethod: str
    customerVaName: str
    returnUrl: str
    callbackUrl: str
    signature: str
    accountLink: AccountLink
    creditCardDetail: CreditCardDetail
    additionalParam: Optional[str] = None
    merchantUserInfo: Optional[str] = None
    phoneNumber: Optional[str] = None
    itemDetails: Optional[List[ItemDetails]] = None
    customerDetail: Optional[CustomerDetail] = None
    expiryPeriod: Optional[int] = None

@dataclass
class CreateTransactionResponse:
    merchantCode: str
    reference: str
    paymentUrl: str
    vaNumber: str
    amount: str
    qrString: str

@dataclass
class GetTransactionStatusRequest():
    merchantCode: str
    merchantOrderId: str
    signature: str

@dataclass
class GetTransactionStatusResponse():
    merchantOrderId: str
    reference: str
    amount: str
    fee: str
    statusCode: str
    statusMessage: str


import unittest
import os

import Duitku

from http import HTTPStatus
from datetime import datetime
from dotenv import load_dotenv

load_dotenv()

class TestInvoice(unittest.TestCase):
    duitku = Duitku.Duitku()

    client = duitku.client
    client.merchant_code = os.getenv('MERCHANT_CODE')
    client.api_key = os.getenv('API_KEY')

    def test_create_invoice_success(self):
        create_invoice_req = {
            "paymentAmount": 10001,
            "merchantOrderId": datetime.now().strftime("%Y%m%d%H%M%S"),
            "productDetails": "test invoice",
            "email": "test@duitku.com",
            "callbackUrl": "https://duitku.com/callback",
            "returnUrl": "https://duitku.com"
        }

        result = self.duitku.invoice.create(create_invoice_req)
        self.assertEqual(result.status_code, HTTPStatus.OK)
        self.assertEqual(result.message['merchantCode'], self.client.merchant_code)
        self.assertIsInstance(result.message['reference'], str)
        self.assertIsNotNone(result.message['paymentUrl'])
        self.assertIsNotNone(result.message['amount'])
        self.assertEqual(result.message['statusCode'], '00')
        self.assertEqual(result.message['statusMessage'], 'SUCCESS')

    def test_create_invoice_bad_request(self):
        create_invoice_req = {}
        result = self.duitku.invoice.create(create_invoice_req)
        self.assertEqual(result.status_code, HTTPStatus.BAD_REQUEST)
        self.assertIsNotNone(result.message)

if __name__ == '__main__':
    unittest.main()


from .client import DuitkuClient
from .invoice import InvoiceService
from .payment import PaymentService
from .transaction import TransactionService

class Duitku:
    def __init__(self):
        """
        Initialize Duitku object.

        Initialize Duitku object by creating its client,
        invoice, payment, and transaction services.

        :return: None
        """
        self.client = DuitkuClient()
        self.invoice = InvoiceService(self.client)
        self.payment = PaymentService(self.client)
        self.transaction = TransactionService(self.client)


import unittest
import os

import Duitku

from http import HTTPStatus
from dotenv import load_dotenv
from datetime import datetime

load_dotenv()

class TestTransaction(unittest.TestCase):
    duitku = Duitku.Duitku()

    client = duitku.client
    client.merchant_code = os.getenv('MERCHANT_CODE')
    client.api_key = os.getenv('API_KEY')

    test_merchant_order_id = datetime.now().strftime("%Y%m%d%H%M%S")

    def test_create_transaction_success(self):
        request_create_transaction = {
            "paymentAmount": 10001,
            "merchantOrderId": self.test_merchant_order_id,
            "productDetails": "test create transaction",
            "email": "test@duitku.com",
            "paymentMethod": "VC",
            "customerVaName": "Test Transaction",
            "callbackUrl": "https://duitku.com/callback",
            "returnUrl": "https://duitku.com"
        }

        result = self.duitku.transaction.create(request_create_transaction)
        self.assertEqual(result.status_code, HTTPStatus.OK)
        self.assertEqual(result.message['merchantCode'], self.client.merchant_code)
        self.assertIsInstance(result.message['reference'], str)
        self.assertIsNotNone(result.message['paymentUrl'])
        self.assertEqual(result.message['statusCode'], '00')
        self.assertEqual(result.message['statusMessage'], 'SUCCESS')

    def test_craete_transaction_unauthorized(self):
        request_create_transaction = {}
        result = self.duitku.transaction.create(request_create_transaction)
        self.assertEqual(result.status_code, HTTPStatus.UNAUTHORIZED)
        self.assertIsNotNone(result.message)

    def test_get_transaction_status(self):
        request_get_trx_status = {
            "merchantOrderId": self.test_merchant_order_id
        }
        result = self.duitku.transaction.get_status(request_get_trx_status)
        self.assertEqual(result.status_code, HTTPStatus.OK)
        self.assertEqual(result.message['merchantOrderId'], self.test_merchant_order_id)
        self.assertIsNotNone(result.message['reference'])
        self.assertIsNotNone(result.message['amount'])
        self.assertIsNotNone(result.message['statusCode'])
        self.assertIsNotNone(result.message['statusMessage'])

if __name__ == '__main__':
    unittest.main()


"""A setuptools based setup module.

See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / 'README.md').read_text(encoding='utf-8')

setup(
    name="Duitku",
    version="0.2.9",
    description="Duitku Python SDK",
    long_description=long_description,
    url="https://github.com/idoyudha/duitku-python",
    author="Ido Yudhatama",
    author_email="idowidya.yudhatama@gmail.com",
    keywords=["duitku", "duitku api", "duitku python"],
    packages=find_packages(include=["Duitku*"]),
    python_requires='>=3.5',
    install_requires=[
        "requests>=2.25.0",
    ],
)


from dataclasses import dataclass
from typing import List

@dataclass
class GetPaymentMethodRequest:
    merchantCode: str
    amount: int
    datetime: str
    signature: str

@dataclass
class PaymentFee:
    paymentMethod: str
    paymentName: str
    paymentImage: str
    totalFee: str

@dataclass
class GetPaymentMethodResponse:
    paymentFee: List[PaymentFee]
    responseCode: str
    responseMessage: str